@@ -20,7 +20,6 @@
  */
 
 #include <glib/gi18n.h>
-#include <glade/glade.h>
 #include <vncdisplay.h>
 #include <gdk/gdkkeysyms.h>
 
@@ -54,6 +53,7 @@ enum
 };
 
 static void open_vnc (VinagreVncTab *vnc_tab);
+static void setup_toolbar (VinagreVncTab *vnc_tab);
 
 static void
 vinagre_vnc_tab_get_property (GObject    *object,
@@ -186,7 +186,13 @@ vinagre_vnc_tab_dispose (GObject *object)
 static void
 vinagre_vnc_tab_constructed (GObject *object)
 {
-  open_vnc (VINAGRE_VNC_TAB (object));
+  VinagreVncTab *vnc_tab = VINAGRE_VNC_TAB (object);
+
+  if (G_OBJECT_CLASS (vinagre_vnc_tab_parent_class)->constructed)
+    G_OBJECT_CLASS (vinagre_vnc_tab_parent_class)->constructed (object);
+
+  setup_toolbar (vnc_tab);
+  open_vnc (vnc_tab);
 }
 
 static void 
@@ -289,14 +295,16 @@ static void
 vnc_auth_unsupported_cb (VncDisplay *vnc, guint auth_type, VinagreVncTab *vnc_tab)
 {
   GString *message;
-  gchar   *name;
+  gchar   *name, *emphasis;
   VinagreTab *tab = VINAGRE_TAB (vnc_tab);
 
   message = g_string_new (NULL);
   name = vinagre_connection_get_best_name (vinagre_tab_get_conn (tab));
+  emphasis = g_strdup_printf ("<i>%s</i>", name);
 
-  g_string_printf (message, _("Authentication method to host <i>%s</i> is unsupported. (%u)"),
-		   name,
+  /* Translators: %s is a host name or IP address; %u is a code error (number). */
+  g_string_printf (message, _("Authentication method to host %s is unsupported. (%u)"),
+		   emphasis,
 		   auth_type);
 
   vinagre_utils_show_error (_("Authentication unsupported"),
@@ -304,6 +312,7 @@ vnc_auth_unsupported_cb (VncDisplay *vnc, guint auth_type, VinagreVncTab *vnc_ta
 			    GTK_WINDOW (vinagre_tab_get_window (tab)));
   g_string_free (message, TRUE);
   g_free (name);
+  g_free (emphasis);
 
   vinagre_tab_remove_from_notebook (tab);
 }
@@ -357,7 +366,6 @@ vnc_initialized_cb (VncDisplay *vnc, VinagreVncTab *vnc_tab)
   gboolean scaling, view_only, fullscreen;
   VinagreTab *tab = VINAGRE_TAB (vnc_tab);
   VinagreConnection *conn = vinagre_tab_get_conn (tab);
-  VinagreWindow *window = vinagre_tab_get_window (tab);
 
   g_object_get (conn,
 		"view-only", &view_only,
@@ -371,9 +379,6 @@ vnc_initialized_cb (VncDisplay *vnc, VinagreVncTab *vnc_tab)
   vnc_display_set_keyboard_grab (VNC_DISPLAY(vnc_tab->priv->vnc), TRUE);
   vnc_display_set_pointer_grab (VNC_DISPLAY(vnc_tab->priv->vnc), TRUE);
 
-  if (fullscreen)
-    vinagre_window_toggle_fullscreen (window);
-
   vinagre_vnc_connection_set_desktop_name (VINAGRE_VNC_CONNECTION (conn),
 					   vnc_display_get_name (VNC_DISPLAY (vnc_tab->priv->vnc)));
 
@@ -415,36 +420,32 @@ ask_credential (VinagreVncTab *vnc_tab,
 		gchar         **username,
 		gchar         **password)
 {
-  GladeXML        *xml;
-  const char      *glade_file;
+  GtkBuilder      *xml;
   GtkWidget       *password_dialog, *host_label, *save_credential_check;
   GtkWidget       *password_label, *username_label, *image;
-  gchar           *name, *label;
+  gchar           *name;
   int             result;
   ControlOKButton control;
   VinagreTab      *tab = VINAGRE_TAB (vnc_tab);
   VinagreConnection *conn = vinagre_tab_get_conn (tab);
 
-  glade_file = vinagre_utils_get_glade_filename ();
-  xml = glade_xml_new (glade_file, NULL, NULL);
+  xml = vinagre_utils_get_builder ();
 
-  password_dialog = glade_xml_get_widget (xml, "auth_required_dialog");
+  password_dialog = GTK_WIDGET (gtk_builder_get_object (xml, "auth_required_dialog"));
   gtk_window_set_transient_for (GTK_WINDOW(password_dialog),
 				GTK_WINDOW(vinagre_tab_get_window (tab)));
 
-  host_label = glade_xml_get_widget (xml, "host_label");
+  host_label = GTK_WIDGET (gtk_builder_get_object (xml, "host_label"));
   name = vinagre_connection_get_best_name (conn);
-  label = g_strdup_printf ("<i>%s</i>", name);
-  gtk_label_set_markup (GTK_LABEL (host_label), label);
+  gtk_label_set_label (GTK_LABEL (host_label), name);
   g_free (name);
-  g_free (label);
 
-  control.uname  = glade_xml_get_widget (xml, "username_entry");
-  control.pw     = glade_xml_get_widget (xml, "password_entry");
-  control.button = glade_xml_get_widget (xml, "ok_button");
-  password_label = glade_xml_get_widget (xml, "password_label");
-  username_label = glade_xml_get_widget (xml, "username_label");
-  save_credential_check = glade_xml_get_widget (xml, "save_credential_check");
+  control.uname  = GTK_WIDGET (gtk_builder_get_object (xml, "username_entry"));
+  control.pw     = GTK_WIDGET (gtk_builder_get_object (xml, "password_entry"));
+  control.button = GTK_WIDGET (gtk_builder_get_object (xml, "ok_button"));
+  password_label = GTK_WIDGET (gtk_builder_get_object (xml, "password_label"));
+  username_label = GTK_WIDGET (gtk_builder_get_object (xml, "username_label"));
+  save_credential_check = GTK_WIDGET (gtk_builder_get_object (xml, "save_credential_check"));
 
   image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_AUTHENTICATION, GTK_ICON_SIZE_BUTTON);
   gtk_button_set_image (GTK_BUTTON (control.button), image);
@@ -619,6 +620,7 @@ vnc_desktop_resize_cb (VncDisplay *vnc, int x, int y, VinagreVncTab *tab)
 {
   g_object_notify (G_OBJECT (tab), "original-width");
   g_object_notify (G_OBJECT (tab), "original-height");
+  g_object_notify (G_OBJECT (tab), "tooltip");
 }
 
 static GSList *
@@ -732,13 +734,19 @@ setup_toolbar (VinagreVncTab *vnc_tab)
   GtkWidget *toolbar = vinagre_tab_get_toolbar (VINAGRE_TAB (vnc_tab));
   GtkWidget *button;
 
+  /* Space */
+  button = GTK_WIDGET (gtk_separator_tool_item_new ());
+  gtk_tool_item_set_expand (GTK_TOOL_ITEM (button), TRUE);
+  gtk_widget_show (GTK_WIDGET (button));
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (button), -1);
+
   /* Scaling */
   button = GTK_WIDGET (gtk_toggle_tool_button_new ());
   gtk_tool_button_set_label (GTK_TOOL_BUTTON (button), _("Scaling"));
   gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (button), _("Scaling"));
   gtk_tool_button_set_icon_name (GTK_TOOL_BUTTON (button), "zoom-fit-best");
   gtk_widget_show (GTK_WIDGET (button));
-  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (button), 0);
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (button), -1);
   g_signal_connect (button, "toggled", G_CALLBACK (scaling_button_clicked), vnc_tab);
   vnc_tab->priv->scaling_button = button;
 
@@ -748,7 +756,7 @@ setup_toolbar (VinagreVncTab *vnc_tab)
   gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (button), _("Read only"));
   gtk_tool_button_set_icon_name (GTK_TOOL_BUTTON (button), "emblem-readonly");
   gtk_widget_show (GTK_WIDGET (button));
-  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (button), 0);
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (button), -1);
   g_signal_connect (button, "toggled", G_CALLBACK (viewonly_button_clicked), vnc_tab);
   vnc_tab->priv->viewonly_button = button;
 
@@ -759,7 +767,7 @@ setup_toolbar (VinagreVncTab *vnc_tab)
   gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (button), _("Send Ctrl-Alt-Del"));
   g_signal_connect (button, "clicked", G_CALLBACK (cad_button_clicked), vnc_tab);
   gtk_widget_show (GTK_WIDGET (button));
-  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (button), 0);
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (button), -1);
 }
 
 static void
@@ -782,10 +790,52 @@ vnc_tab_clipboard_cb (GtkClipboard *cb, GdkEvent *event, VinagreVncTab *vnc_tab)
   g_free (text);
 }
 
+/*
+ * Called when the main container widget's size has been set.
+ * It attempts to fit the VNC widget into this space while
+ * maintaining aspect ratio
+ *
+ * Code borrowed from from virt-viewer, thanks Daniel Berrange :)
+ */
+static void
+vnc_tab_resize_align (GtkWidget *widget,
+		      GtkAllocation *alloc,
+		      VinagreVncTab *vnc_tab)
+{
+  double desktopAspect = (double)vnc_display_get_width (VNC_DISPLAY (vnc_tab->priv->vnc)) / (double)vnc_display_get_height (VNC_DISPLAY (vnc_tab->priv->vnc));
+  double scrollAspect = (double)alloc->width / (double)alloc->height;
+  int height, width;
+  GtkAllocation child;
+  int dx = 0, dy = 0;
+
+  if (!vnc_display_is_open (VNC_DISPLAY (vnc_tab->priv->vnc)))
+    return;
+
+  if (scrollAspect > desktopAspect)
+    {
+      width = alloc->height * desktopAspect;
+      dx = (alloc->width - width) / 2;
+      height = alloc->height;
+    }
+  else
+    {
+      width = alloc->width;
+      height = alloc->width / desktopAspect;
+      dy = (alloc->height - height) / 2;
+    }
+
+  child.x = alloc->x + dx;
+  child.y = alloc->y + dy;
+  child.width = width;
+  child.height = height;
+  gtk_widget_size_allocate(vnc_tab->priv->vnc, &child);
+}
+
 static void
 vinagre_vnc_tab_init (VinagreVncTab *vnc_tab)
 {
   GtkClipboard *cb;
+  GtkWidget *align;
 
   vnc_tab->priv = VINAGRE_VNC_TAB_GET_PRIVATE (vnc_tab);
   vnc_tab->priv->clipboard_str = NULL;
@@ -794,9 +844,13 @@ vinagre_vnc_tab_init (VinagreVncTab *vnc_tab)
 
   /* Create the vnc widget */
   vnc_tab->priv->vnc = vnc_display_new ();
-  vinagre_tab_add_view (VINAGRE_TAB (vnc_tab), vnc_tab->priv->vnc);
+  align = gtk_alignment_new (0.5, 0.5, 1, 1);
 
-  setup_toolbar (vnc_tab);
+  g_signal_connect(align, "size-allocate",
+		  G_CALLBACK (vnc_tab_resize_align), vnc_tab);
+  gtk_container_add (GTK_CONTAINER (align), vnc_tab->priv->vnc);
+
+  vinagre_tab_add_view (VINAGRE_TAB (vnc_tab), align);
 
   g_signal_connect (vnc_tab->priv->vnc,
 		    "vnc-connected",
@@ -888,16 +942,22 @@ void
 vinagre_vnc_tab_paste_text (VinagreVncTab *tab, const gchar *text)
 {
   gchar *out;
-  size_t a, b;
+  gsize a, b;
+  GError *error = NULL;
+
   g_return_if_fail (VINAGRE_IS_VNC_TAB (tab));
 
-  out = g_convert (text, -1, "iso8859-1", "utf-8", &a, &b, NULL);
-
+  out = g_convert_with_fallback (text, -1, "iso8859-1//TRANSLIT", "utf-8", NULL, &a, &b, &error);
   if (out)
     {
       vnc_display_client_cut_text (VNC_DISPLAY (tab->priv->vnc), out);
       g_free (out);
     }
+  else
+    {
+      g_warning ("%s", error->message);
+      g_error_free (error);
+    }
 }
 
 gboolean
