@@ -22,7 +22,7 @@
 #include <glib/gi18n.h>
 #include "vinagre-utils.h"
 
-#define VINAGRE_GLADE_FILE  "vinagre.glade"
+#define VINAGRE_UI_FILE  "vinagre.ui"
 #define VINAGRE_UI_XML_FILE "vinagre-ui.xml"
 
 #ifdef HAVE_CONFIG_H
@@ -119,12 +119,12 @@ vinagre_utils_toggle_widget_visible (GtkWidget *widget)
 }
 
 const gchar *
-vinagre_utils_get_glade_filename (void)
+vinagre_utils_get_ui_filename (void)
 {
-  if (g_file_test (VINAGRE_GLADE_FILE, G_FILE_TEST_EXISTS))
-    return VINAGRE_GLADE_FILE;
+  if (g_file_test (VINAGRE_UI_FILE, G_FILE_TEST_EXISTS))
+    return VINAGRE_UI_FILE;
   else
-    return VINAGRE_DATADIR "/" VINAGRE_GLADE_FILE;
+    return VINAGRE_DATADIR "/" VINAGRE_UI_FILE;
 }
 
 const gchar *
@@ -136,6 +136,26 @@ vinagre_utils_get_ui_xml_filename (void)
     return VINAGRE_DATADIR "/" VINAGRE_UI_XML_FILE;
 }
 
+GtkBuilder *
+vinagre_utils_get_builder (void)
+{
+  GtkBuilder *xml;
+  GError     *error = NULL;
+
+  xml = gtk_builder_new ();
+  if (!gtk_builder_add_from_file (xml,
+				  vinagre_utils_get_ui_filename (),
+				  &error))
+    {
+      g_error (_("Could not load builder file: %s"), error->message);
+      g_error_free (error);
+      g_object_unref (xml);
+      return NULL;
+    }
+
+  return xml;
+}
+
 /*
  * Doubles underscore to avoid spurious menu accels.
  */
@@ -183,7 +203,7 @@ static void _default_log (const gchar *log_domain G_GNUC_UNUSED,
 			 const gchar *message,
 			 gpointer user_data G_GNUC_UNUSED)
 {
-  printf ("gtk-vnc: %s", message);
+  printf ("gtk-vnc: %s\n", message);
 }
 
 void
